# Copyright 2012, Dag Wieers <dag@wieers.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.errors import AnsibleError
from ansible.playbook.conditional import Conditional
from ansible.plugins.action import ActionBase


class ActionModule(ActionBase):
    ''' Fail with custom message '''

    TRANSFERS_FILES = False

    def run(self, tmp=None, task_vars=None):
        if task_vars is None:
            task_vars = dict()

        result = super(ActionModule, self).run(tmp, task_vars)

        if 'that' not in self._task.args:
            raise AnsibleError('conditional required in "that" string')

        msg = None
        if 'msg' in self._task.args:
            msg = self._task.args['msg']

        # directly access 'that' via untemplated args from the task so we can intelligently trust embedded
        # templates and preserve the original inputs/locations for better messaging on assert failures and
        # errors.
        # FIXME: even in devel, things like `that: item` don't always work properly (truthy string value
        # is not really an embedded expression)
        # we could fix that by doing direct var lookups on the inputs
        # FIXME: some form of this code should probably be shared between debug, assert, and
        # Task.post_validate, since they
        # have a lot of overlapping needs
        try:
            thats = self._task.untemplated_args['that']
        except KeyError:
            # in the case of "we got our entire args dict from a template", we can just consult the
            # post-templated dict (the damage has likely already been done for embedded templates anyway)
            thats = self._task.args['that']

        # FIXME: this is a case where we only want to resolve indirections, NOT recurse containers
        # (and even then, the leaf-most expression being wrapped is at least suboptimal
        # (since its expression will be "eaten").
        if isinstance(thats, str):
            thats = self._templar.template(thats)

        # make sure the 'that' items are a list
        if not isinstance(thats, list):
            thats = [thats]

        # Now we iterate over the that items, temporarily assigning them
        # to the task's when value so we can evaluate the conditional using
        # the built in evaluate function. The when has already been evaluated
        # by this point, and is not used again, so we don't care about mangling
        # that value now
        cond = Conditional(loader=self._loader)
        for that in thats:
            cond.when = [that]
            test_result = cond.evaluate_conditional(templar=self._templar, all_vars=task_vars)
            if not test_result:
                result['failed'] = True
                result['evaluated_to'] = test_result
                result['assertion'] = that

                if msg:
                    result['msg'] = msg

                return result

        result['changed'] = False
        result['msg'] = 'all assertions passed'
        return result
